package DBIx::Class::InflateColumn::TimeMomentX_BOKUTIN;
$DBIx::Class::InflateColumn::TimeMomentX_BOKUTIN::VERSION = '0.17';
use strict;
use warnings;
use base qw/DBIx::Class/;
use DBIx::Class::Carp;
use Time::MomentX::BOKUTIN;
use namespace::clean;

sub register_column {
  my ($self, $column, $info, @rest) = @_;

  $self->next::method($column, $info, @rest);

  my $data_type = $info->{data_type} || '';
  return unless $data_type =~ /^(date|datetime|timestamp|timestamp with time zone)$/;

  my $with_zone = $data_type =~ /with time zone/ ? 1 : 0;

  $self->inflate_column(
    $column =>
      {
        inflate => sub {
          my ($value, $obj) = @_;

          local $@;
          my $tm = eval { tmx($value) };
          $self->throw_exception("Error while inflating '$value' for $column on ${self}. error:$@") if $@;
          $tm;
        },
        deflate => sub {
          # my ($value, $obj) = @_;

          my $tm = $_[0]->isa('Time::Moment') ? $_[0] : tmx( from_object => $_[0] );

          if ($with_zone) {
            $tm->to_string;
          }
          else {
            # offset は静かに棄てられてしまうので日時を変換しておきます。
            # with_system_offset_same_instant は tzset(2) に従うだけなので show timezone と一致しているかは保証しません。
            #
            # 変換のコストを避けたい場合は、事前に文字列して、ここを通らないようにしてください。
            #
            # 既に system_offset になっているか確認して呼ばない分岐を入れる必要が無いくらい速いはずなので、ただ呼ぶだけにする
            $tm->with_system_offset_same_instant->to_string;
          }
        },
      }
  );
}

1;

=pod

=encoding UTF-8

=head1 NAME

DBIx::Class::InflateColumn::TimeMomentX_BOKUTIN

=cut

__END__

DBIx::Class::InflateColumn::DateTime はすばらしい

    DBIx::Class::InflateColumn::DateTime はとても柔軟に出来ていますが、ここでは色々省きます。

postgres の動き

    without timezone
        取得時
            offset は付きません。
                2018-01-01 00:00:00
        保存時
            Z や +09:00 といった offset は静かに棄てられます！
            show timezone に変換してくれたりはしないようです。
                2018-01-01T00:00:00+09:00 -> 2018-01-01 00:00:00
                2018-01-01T00:00:00Z      -> 2018-01-01 00:00:00

            https://www.postgresql.jp/document/9.5/html/datatype-datetime.html
            timestamp without time zoneと決定済みのリテラルでは、PostgreSQLは警告なく時間帯情報をすべて無視します。 つまり、結果の値は明示された入力値の日付/時刻フィールドから持ち込まれますが、時間帯の調整はなされません。

    with timezone
        取得時
            set timezone の offset が付きます。
                2018-01-01 00:00:00+09:00
        保存時
            offset を解釈して、内部的には UTC で保存されるようです。

この InflateColumn では

    without time zone でも show timezone を確認しません。

    ただ tmx に委ねます。

備考

    show timezone と tzset(2) が一致しないケースは、今のところサポートしていません。
    その必要がある場合には show timezone と tzset(2) を一致させた上 without -> with などを検討してください。
    https://stackoverflow.com/questions/9772825/postgresql-alter-type-timestamp-without-time-zone-with-time-zone
