use Test::More;

use utf8;
use strict;
use warnings;
use feature qw(:5.10);
use POSIX qw(:locale_h);
use Time::Local ();

require_ok "Time::MomentX::BOKUTIN";

my $class = Time::MomentX::BOKUTIN::tmx();
sub tmx { Time::MomentX::BOKUTIN::tmx(@_) }

{
    $ENV{TZ} = 'Asia/Tokyo';
    POSIX::tzset();
    my $tm1 = $class->from_fuzzy('2018-01-01');
    is $tm1->offset, 9*60;
}

{
    $ENV{TZ} = 'UTC';
    POSIX::tzset();
    my $tm1 = $class->from_fuzzy('2018-01-01');
    is $tm1->offset, 0;
}

{
    my $tm1 = $class->from_string('2018-01-01T00:00:00Z');

    setlocale(LC_TIME, 'ja_JP.UTF-8');
    is $tm1->strftime('%a'), '月';
    is $tm1->strftime('%A'), '月曜日';

    is $tm1->strftime('%a','en'), 'Mon';
    is $tm1->strftime('%A','en'), 'Monday';

    is $tm1->strftime('%a','zh'), '一';
    is $tm1->strftime('%A','zh'), '星期一';

    setlocale(LC_TIME, 'C');
    is $tm1->strftime('%a'), 'Mon';
    is $tm1->strftime('%A'), 'Monday';
}

{
    my $tm1 = $class->from_string('2018-01-01T00:00:00Z');
    my $tm2 = $class->from_string('2018-01-01T00:00:00Z');
    if ($tm1 == $tm2) {
        ok 'same';
    }
    unless ($tm1 > $tm2) {
        ok 'gt'
    }
    unless ($tm1 < $tm2) {
        ok 'lt'
    }
}

{
    require DateTime;
    my $dt = DateTime->now;
    my $tm = $class->from_object($dt);
    is ref($tm), $class;
}

{
    $ENV{TZ} = 'UTC';
    POSIX::tzset();
    is Time::Moment->from_rd(1), "0001-01-01T00:00:00Z";
    is tmx( from_rd => 1 ),      "0001-01-01T00:00:00Z";

    $ENV{TZ} = 'Asia/Tokyo';
    POSIX::tzset();
    is $class->from_rd(2), "0001-01-02T00:00:00Z";
    is $class->from_rd(2, offset => 9*60), "0001-01-02T00:00:00+09:00";
    is $class->from_rd(2)->with_system_offset_same_instant, "0001-01-02T09:18:00+09:18";
    is $class->from_rd(2)->with_system_offset_same_local,   "0001-01-02T00:00:00+09:18";
    is Time::Moment->from_rd(1, offset => 9*60), "0001-01-01T00:00:00+09:00";
    is tmx( from_rd => 2 ),      "0001-01-02T00:00:00+09:18"; # "0001-01-01T09:18:00+09:18" とはしない

    # https://metacpan.org/source/DROLSKY/DateTime-TimeZone-2.23/lib/DateTime/TimeZone/Asia/Tokyo.pm#PDateTime::TimeZone::Asia::Tokyo
    my $lmt_offset_sec = 33539;
    is tmx( from_rd => 1 )->offset, int $lmt_offset_sec/60;
}

{
    $ENV{TZ} = 'Asia/Tokyo';
    POSIX::tzset();

    my $tm1 = $class->from_string('2018-01-01T00:00:00Z');
    my $tm2 = $class->from_string('2018-01-01T12:00:00Z');
    is $tm1->rd, "736695";
    is $tm2->rd, "736695.5";
    
    is tmx( from_rd => "736695",   offset => 0 ), '2018-01-01T00:00:00Z';
    is tmx( from_rd => "736695"                ), '2018-01-01T00:00:00+09:00'; # '2018-01-01T09:00:00+09:00' とはしない
    is tmx( from_rd => "736695.5", offset => 0 ), '2018-01-01T12:00:00Z';
    is tmx( from_rd => "736695.5"              ), '2018-01-01T12:00:00+09:00';
}

{
    $ENV{TZ} = 'Asia/Tokyo';
    POSIX::tzset();
    my $tm1 = $class->from_fuzzy('20161125T20:40:17');
    is "$tm1", "2016-11-25T20:40:17+09:00";
}

done_testing;
